#include "checksum.h"


//////// Simple Transfer checksum ////////
// Assembly rips by Coconut Emulator

BYTE simpleChecksum(char * buffer, Uint32 len)
{
	Uint32 i;
	BYTE c = 0;

	for (i = 0; i < len; i++)
	{
		c ^= *buffer++;
	}

	return c;
}

BYTE simpleChecksum(BYTE * buffer, Uint32 len)
{
	Uint32 i;
	BYTE c = 0;

	for (i = 0; i < len; i++)
	{
		c ^= *buffer++;
	}

	return c;
}


//////// File checksum ////////
// This section made possible by Snrrrub

Uint32 getFileChecksum(BYTE * buffer, long * dictionary, Uint32 len)
{
	Uint32 Index = 0,
		   Key   = Index - 1;

	for (Uint32 i = 0; i < len; i++)
	{
		Index	= dictionary[(Key & 255) ^ buffer[i]];
		Key		= (Key >> 8) ^ Index;
	}

	return ~Key;
}


void generate4(long * offset, long key)
{
	offset[0] = key;
	offset[1] = key ^ G4_MODIFIER;
	offset[2] = key ^= (G4_MODIFIER << 1);
	offset[3] = key ^ G4_MODIFIER;
}

void generate16(long * offset, long key)
{
	generate4(offset, key);
	generate4(offset + 4, key  ^ G16_MODIFIER);
	generate4(offset + 8, key  ^= (G16_MODIFIER << 1));
	generate4(offset + 12, key ^ G16_MODIFIER);
}

void generate64(long * offset, long key)
{
	generate16(offset, key);
	generate16(offset + 16, key ^ G64_MODIFIER);
	generate16(offset + 32, key ^= (G64_MODIFIER << 1));
	generate16(offset + 48, key ^ G64_MODIFIER);
}

void generateDictionary(long * offset, long key)
{
	generate64(offset, key);
	generate64(offset + 64, key  ^ G256_MODIFIER);
	generate64(offset + 128, key ^= (G256_MODIFIER << 1));
	generate64(offset + 192, key ^ G256_MODIFIER);
}


//////// Security checksum ////////
//  Supreme thanks go to Coconut Emulator and Snrrrub
//  for providing the assembly rips.

Uint32 generateLevelChecksum(Uint32 key, char * mapData)
{
	long EAX, ECX, ESI, EDX;
	long HighBit;
	long cnt;
	long original_key = key;

	ECX = key;

	if (ECX & 0x80000000)
		HighBit = -1;
	else
		HighBit = 0;

	ECX ^= HighBit;
	ECX -= HighBit;
	ECX &= 0x1F;
	ECX ^= HighBit;
	ECX -= HighBit;

	if (ECX >= 0x400)
		return key;

	EDX = key % 0x1F;
	ESI = (ECX << 10) + (long)mapData + EDX;
	EAX = 0x400 - EDX;
	cnt = (0x41F - ECX) >> 5;
	mapData = (char *)EAX;

	for(; cnt > 0; cnt--, ESI += 0x8000)
	{
		EDX = EAX + ESI;
		ECX = ESI;

		if (ESI >= EDX)
			continue;

		while (ECX < EDX)
		{
			BYTE tile = (*((char *)ECX));

			if (tile != vieNoTile)
			{
				if ((tile < vieNormalEnd) || (tile == vieSafeZone))
					key += original_key ^ tile;
			}

			ECX += 0x1F;
		}

		EAX = (long)mapData;
	}

	return key;
}

Uint32 generateParameterChecksum(Uint32 key, Uint32 * settings)
{
	Uint32 i, c = 0;

	for (i = 0; i < 357; i++)
	{
		c += settings[i] ^ key;
	}

	return c;
}

Uint32 generatePartialEXE(Uint32 key, char * buffer, Uint32 len)
{
	Uint32 data,
		   partialKey = 0,
		   offset = 0;

	while (len > 4)
	{
		data = getLong(buffer, offset);

		if ((len & 0x7FF) == 0)
		{
			partialKey += (data | key);
		}
		else
		{
			partialKey ^= (data ^ key);
		}

		len -= 3;
		offset += 3;
	}

	delete []buffer;

	return partialKey;
}

Uint32 generateEXEChecksum(Uint32 key, char * fileName)
{
	fileAccess fa;
	char * fd;

	if (fa.open(fileName))
	{
		Uint32 c = 0;

		c += generatePartialEXE(key, fd = fa.readStuff(0x00006940, 0x0000D5A0), 0x0000D5A0);	delete []fd;
		c += generatePartialEXE(key, fd = fa.readStuff(0x00000400, 0x00002480), 0x00002480);	delete []fd;
		c += generatePartialEXE(key, fd = fa.readStuff(0x0001CAE0, 0x00006F70), 0x00006F70);	delete []fd;
		c += generatePartialEXE(key, fd = fa.readStuff(0x00042E70, 0x000021E0), 0x000021E0);	delete []fd;
		c += generatePartialEXE(key, fd = fa.readStuff(0x000305A0, 0x0000D3C0), 0x0000D3C0);	delete []fd;
		c += generatePartialEXE(key, fd = fa.readStuff(0x0002A5B0, 0x00005FE0), 0x00005FE0);	delete []fd;
		c += generatePartialEXE(key, fd = fa.readStuff(0x00003960, 0x00001C90), 0x00001C90);	delete []fd;
		c += generatePartialEXE(key, fd = fa.readStuff(0x00013EF0, 0x00008BE0), 0x00008BE0);	delete []fd;
		c += generatePartialEXE(key, fd = fa.readStuff(0x00005750, 0x000000E0), 0x000000E0);	delete []fd;

		return c;
	}
	else
	{
		return -1;
	}
}
