/*
	Simple socket wrapper by Catid@pacbell.net
	------------------------------------------
	UDPSocket supports polling and writing.
	Before you can use these features, you must
	first call beginWinsock().
*/

#ifndef SOCKETS_H
#define SOCKETS_H

#include <winsock2.h>

#include <stdio.h>


void WriteLog(char * Packet, int Length);

void ReadLog(char * Packet, int Length);


void beginWinsock();					// Register process with winsock2

void endWinsock();						// Unregister process from winsock2


struct INADDR
{
	WORD family;						// Socket family (Always INET_ADDR)
	WORD port;							// Socket subclassing
	DWORD ip;							// Host address
	BYTE padding[8];					// Unused space

	INADDR(DWORD ip, WORD port);		// Fill address
	INADDR();							// Fill address

	bool operator==(INADDR other);		// Validate source
	bool operator==(INADDR * other);	// Validate source

	void operator=(INADDR other);		// Copy source
	void operator=(INADDR * other);		// Copy source

	char * getString();					// Dotted quad representation
	int getPort();						// Return host-order
	sockaddr * getAddress();			// Passable address
	void set(DWORD ip, WORD port);		// Re-fill address
};


class UDPPacket
{
public:
	INADDR	src;						// Sender address
	char *	msg;						// Message contents
	int		len;						// Message length

	UDPPacket(INADDR src,
			  BYTE * msg,
			  int len);					// Fill the Packet
	~UDPPacket();						// Clean up
};


class UDPSocket
{
	SOCKET sid;							// Socket identifier
	HANDLE event;						// Read event polling
	INADDR remote;						// Remote address

public:
	UDPSocket();						// Start up
	~UDPSocket();						// Clean up

	int create(WORD port);				// Bind to a port

	void set(DWORD ip, WORD port);		// Set host
	void set(INADDR host);				// Set host

	bool send(BYTE * msg, int len);		// Send a datagram
	bool send(char * msg, int len);		// Send a datagram

	UDPPacket * poll();					// Recv a datagram
};


char * WSAGetErrorString(int code);		// Get a string-representation of an error code


DWORD resolveHostname(char * name);		// Resolve inet_addr from dotted quad representations

#endif	// SOCKETS_H
